"""Models module."""

__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
	from _ohlcv import Ohlcv
	from _study import Study, StudyConfig
	from _indicator import Indicator, IndicatorInfo, IndicatorParameter, MinMaxThreshold

__all__ = [
	"Ohlcv",
	"Study",
	"StudyConfig",
	"Indicator",
	"IndicatorInfo",
	"MinMaxThreshold",
	"IndicatorParameter"
]

__cache = {}

def _build_path(string: str) -> str:
	if any(x.lower() in string.lower() for x in ['indicator','threshold']):
		return '._indicator'
	elif "study" in string.lower():
		return '._study'
	elif "ohlcv" in string.lower():
		return '._ohlcv'

def __getattr__(string):
	try:
		return __cache[string]
	except KeyError:
		pass

	if string not in __all__:
		raise ImportError(f"Package 'sphoin.models' has no class '{string}'")

	module = import_module(_build_path(string), package="sphoin.models")
	class__ = getattr(module, string)

	__cache[string] = class__
	return class__


from textual.reactive import reactive
from textual.widgets import Static
from rich.text import Text
from ...app import Slot
from ...plot import Plot

class Time(Static):

	slot : [Slot, None] = reactive(None)
	brightness: bool = reactive(False)
	banner : Text = reactive(Text(""))
	dark: bool = reactive(True)

	def __init__(self, slot: Slot) -> None:
		self._slot = slot
		super().__init__()

	def paint_time(self, slot: Slot, dark: bool, brightness: bool) -> Text:
		if self.size.width<50:
			return ''
		return Plot.time_bar(slot=slot,sizeX=self.size.width,dark_theme=dark,brightness=brightness)

	def on_mount(self) -> None:
		self.slot = self._slot
		self.set_interval(1, self.refresh_banner)

	def refresh_banner(self) -> None:
		self.banner = self.paint_time(slot=self.slot, dark=self.dark, brightness=self.brightness)

	def watch_brightness(self, brightness: bool) -> None:
		self.banner = self.paint_time(slot=self.slot,dark=self.dark,brightness=brightness)

	def watch_banner(self, banner: Text) -> None:
		self.update(banner)

	def watch_slot(self, slot: Slot) -> None:
		if slot!=None:
			self.banner = self.paint_time(slot=slot,dark=self.dark,brightness=self.brightness)

	def watch_dark(self, dark: bool) -> None:
		self.banner = self.paint_time(slot=self.slot,dark=dark,brightness=self.brightness)


from rich.text import Text
from rich.style import Style


uni_chars = {
	"space": u'\u0020',
	"vertical_bar":u'\u2524',
	"light":{
		"curved":{
			"horizontal": u'\u2500',
			"vertical": u'\u2502',
			"up_right": u'\u2570',
			"down_right": u'\u256D',
			"left_down": u'\u256E',
			"left_up": u'\u256F',
		},
		"angular":{
			"horizontal": u'\u2500',
			"vertical": u'\u2502',
			"up_right": u'\u2514',
			"down_right": u'\u250C',
			"left_down": u'\u2510',
			"left_up": u'\u2518'
		},
	},
	"heavy":{
		"angular":{
			"horizontal": u'\u2501',
			"vertical": u'\u2503',
			"up_right": u'\u2517',
			"down_right": u'\u250F',
			"left_down": u'\u2513',
			"left_up": u'\u251B'
		},
		"cross":{
			"full":{
				"light_horizontal": u'\u2542',
				"light_vertical": u'\u253F',
				"light_up_right": u'\u2545',
				"light_left_down": u'\u2544',
				"light_left_up": u'\u2546',
				"light_down_right": u'\u2543'
			},
			"vertical":{
				"light_right": u'\u2520',
				"light_left": u'\u2528',
				"left_down": u'\u252A',
				"left_up": u'\u2529',
				"up_right": u'\u2521',
				"down_right": u'\u2522'
			},
			"horizontal":{
				"light_up": u'\u2537',
				"light_down": u'\u252F',
				"left_down": u'\u2531',
				"left_up": u'\u2539',
				"up_right": u'\u253A',
				"down_right": u'\u2532'
			}
		}
	}
}

footer = {
	"display": {
		"left":{
			"s": "{market}{interval}|{max_flag}|{min_flag}|",			
			"m": "{market} | {interval}| {max_flag}| {min_flag}|",
			"l": "{exchange}|{market}|{interval}|{max_flag}|{min_flag}|",
			"xl":" {exchange} | {market} | {interval} | {max_flag} | {min_flag} |"
			},
		"mid":{
			"s": "|h|q|r|t|s",
			"m": "| h | q | r | t | s ",
			"l": "|h Help|q Quit|r Refresh|t Theme|s Signal",
			"xl":"| h Help | q Quit | r Refresh | t Theme | s Signal ",
			},
		"right":{
			"s": "|{eta}",
			"m": "| {eta} ",
			"l": "|ETA {eta}",
			"xl":"| ETA {eta} "
			}
	},
	"color":{
		"exchange":{
			"color":"black",
			"bgcolor":"yellow"
		},
		"market":{
			"color":"black",
			"bgcolor":"white"
		},
		"interval":{
			"color":"black",
			"bgcolor":"white"
		},
		"max_flag":{
			"color":"black",
			"bgcolor":"yellow"
		},
		"min_flag":{
			"color":"black",
			"bgcolor":"yellow"
		},
	},
	"actions": {
		"help":[" h Help ","h Help"," h ","h"],
		"quit":[" q Quit ","q Quit"," q ","q"],
		"refresh":[" r Refresh ","r Refresh"," r ","r"],
		"theme":[" t Theme ","t Theme"," t ","t"],
		"signal":[" s Signal ","s Signal"," s ","s"]
	}
}

def intersperse(_list: list, item):
	result = [item] * (len(_list) * 2 -1)
	result[0::2] = _list
	return result

def text_with_style(
	string: str,
	fg: str,
	bg: str) -> tuple:
	return [string,f"{fg} on {bg}"]

def isNumber(n) -> bool:
	return not n == None



class Indicator:
	"""
Indicator class
	Technical indicator or pattern

Attributes
----------
json : dict
name : str
info : IndicatorInfo
min_threshold : MinMaxThreshold
max_threshold : MinMaxThreshold
parameters : list
	List of IndicatorParameters
raw : list
	List of raw values of technical indicator or pattern
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.name : str = kwargs['name']
		self.info : IndicatorInfo = IndicatorInfo(**kwargs['info'])
		self.min_threshold : MinMaxThreshold = MinMaxThreshold(**kwargs['min_threshold'])
		self.max_threshold : MinMaxThreshold = MinMaxThreshold(**kwargs['max_threshold'])
		self.parameters : list = [IndicatorParameter(**kwargs[x]) for x in kwargs.keys() if x not in ['min_threshold','max_threshold','raw','name','info']]
		self.raw : list = list(kwargs['raw'].values())

class IndicatorInfo:
	"""
IndicatorInfo
	Detailed info of technical indicator or pattern

Attributes
----------
json : dict
display_name : str
name : str
group : str
function_flags : list
input_names : dict
output_flags : dict
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.display_name : str = kwargs['display_name']
		self.name : str = kwargs['name']
		self.group : str = kwargs['group']
		self.function_flags : list = kwargs['function_flags']
		self.input_names : dict = kwargs['input_names']
		self.output_flags : dict = kwargs['output_flags']

class MinMaxThreshold:
	"""
MinMaxThreshold class
	Used for study code #2

Attributes
----------
json : dict
display_name : str
value : int
	Custom value to filter indicator raw values
default_value : int
	Values used if custom value is not set
help : str
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.display_name : str = kwargs['display_name']
		self.default_value : int = int(kwargs['default_value'])
		self.help : str = kwargs['help']
		self.value : int = kwargs['value']

class IndicatorParameter:
	"""
IndicatorParameter class

Attributes
----------
json : dict
name : str
display_name : str
help : str
type : int
	Type value of technical indicator or pattern
	values : 
		0 - float value
		2 - int value
		3 - int value
value : float or int
default_value : float or int
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.name : str = kwargs['name']
		self.display_name : str = kwargs['display_name']
		self.help : str = kwargs['help']
		self.type : int = int(kwargs['type'])
		if self.type == 0:
			self.default_value : float = kwargs['default_value']
			self.value : float = kwargs['value']
		elif self.type == 2 or self.type == 3:
			self.default_value : int = kwargs['default_value']
			self.value : int = kwargs['value']



from ._indicator import Indicator

class Study:
	"""
Study class

Attributes
----------
crt : str
	Study index number
name : str
	Study custom name
study : str
	Study name
	['U|D','M|M','X','---','[ ]']
description : str
	Study description
code : str
	Study code
	['#1','#2','#3','#4','#5']
normalize : bool
	True if values are normalized and False if not
inverse : bool
	True if resulting signals are inversed and False is not
config : dict
	StudyConfig
result : list
	List of resulted signals
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.crt : str = kwargs['crt']
		self.name : str = kwargs['name']
		self.study : str = kwargs['study']
		self.code : str = kwargs['code']
		self.description : str = kwargs['description']
		self.normalize : bool = kwargs['normalize']
		self.inverse : bool = kwargs['inverse']
		self.config : dict = StudyConfig(**kwargs['config'])
		self.result : list = list(kwargs['result'].values())

class StudyConfig:
	"""
StudyConfig class

Attributes
----------
indicator_1: Indicator
indicator_2: Indicator
	None: If the Study uses only 1 technical indicator or pattern
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.indicator_1 : Indicator = None
		self.indicator_2 : Indicator = None
		for x in kwargs.keys():
			if x not in ['raw','PASSS']:
				if self.indicator_1 == None:
					data = kwargs[x]
					data['name'] = x
					data['raw'] = kwargs['raw'][x]
					self.indicator_1 : Indicator = Indicator(**data)
				elif self.indicator_2 == None:
					data = kwargs[x]
					data['name'] = x
					data['raw'] = kwargs['raw'][x]
					self.indicator_2 : Indicator = Indicator(**data)
			elif x == 'PASSS':
				self.indicator_2 : Indicator = None


"""Top-level package for sphoin."""

__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"


from textual.reactive import reactive
from textual.widgets import Static
from rich.text import Text
from rich.style import Style
from ...app import Slot
from ...plot import Plot
from ...utils import intersperse


class Graph(Static):

	plot : Text = reactive(Text(""))
	slot : [Slot, None] = reactive(None)
	plot_type : [str, None] = reactive(None)
	dark : bool = reactive(True)
	brightness : bool = reactive(False)
	signal_as_line : bool = reactive(True)

	def __init__(self, slot_init: Slot, plot_type: str) -> None:
		self._slot = slot_init
		self._plot_type = plot_type
		super().__init__()


	def paint_plot(self, slot: Slot, plot_type: str, dark: bool, brightness: bool) -> str:
		if self.size.width<50:
			return f"Screen width too small: {self.size.width}\nMin: 50"
		elif self.size.width>500:
			return f"Screen width too big: {self.size.width}\nMax: 500"
		if self.size.height<4:
			return f"Screen height too small: {self.size.height}\nMin: 4"
		elif self.size.height>200:
			return f"Screen height too big: {self.size.height}\nMax: 200"
		if slot != None:
			try:
				if plot_type == 'line':
					new_plot = Plot.line(
						series=slot.ohlcv.close,
						signals=slot.signals_sum,
						min_flag=slot.min_flag,
						max_flag=slot.max_flag, 
						color1=self.slot.colors['1'], 
						color2=self.slot.colors['-1'],
						sizeX=self.size.width,
						sizeY=self.size.height,
						signal_as_line=self.signal_as_line,
						dark_theme=dark,
						brightness=brightness)
					return new_plot
				elif plot_type == 'studies':
					new_plot = []
					for study in slot.studies:
						new_plot.append(Plot.study_bar(
						study=study,
						color1=slot.colors['1'],
						color2=slot.colors['-1'],
						sizeX=self.size.width,
						dark_theme=dark,
						brightness=brightness
						))
					new_plot = intersperse(new_plot,Text("\n"))
					return Text.assemble(*tuple(new_plot))
				elif plot_type == 'signals':
					new_plot = Plot.line(
						series=slot.signals_sum,
						signals=slot.signals_sum,
						min_flag=slot.min_flag,
						max_flag=slot.max_flag, 
						color1=slot.colors['1'], 
						color2=slot.colors['-1'],
						sizeX=self.size.width,
						sizeY=self.size.height,
						signal_as_line=self.signal_as_line,
						dark_theme=dark,
						brightness=brightness)
					return new_plot
			except Exception as e:
				return Text(f"{e}")
		else:
			return Text("Painting...")


	def on_mount(self) -> None:
		self.slot = self._slot
		self.plot_type = self._plot_type
		self.set_interval(1,self.refresh_plot)

	def refresh_plot(self) -> None:
		self.plot = self.paint_plot(slot=self.slot,plot_type=self.plot_type,dark=self.dark, brightness=self.brightness)

	def watch_plot(self, plot: Text) -> None:
		self.update(plot)

	def watch_brightness(self, brightness: bool) -> None:
		self.brightness = brightness
		self.plot = self.paint_plot(slot=self.slot,plot_type=self.plot_type, dark=self.dark, brightness=brightness)

	def watch_dark(self, dark: bool) -> None:
		self.dark = dark
		self.plot = self.paint_plot(slot=self.slot,plot_type=self.plot_type, dark=dark, brightness=self.brightness)

	def watch_slot(self, slot: str) -> None:
		self.plot = self.paint_plot(slot=slot,plot_type=self.plot_type, dark=self.dark, brightness=self.brightness)

	def watch_plot_type(self, plot_type: str) -> None:
		self.plot = self.paint_plot(slot=self.slot,plot_type=plot_type, dark=self.dark, brightness=self.brightness)



"""Plot module."""

__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

import typing
from importlib import import_module
from rich.text import Text

if typing.TYPE_CHECKING:
	from ._plot import Plot
	from ._utils import isNumber, uni_chars, intersperse

__all__ = [
	"Plot"
]

__cache = {}

def _build_path(string: str) -> str:
	if "plot" in string.lower():
		return '._plot'
	elif any(x.lower() in string.lower() for x in ['isNumber','uni_chars','text_with_style']):
		return '._utils'

def __getattr__(string):
	try:
		return __cache[string]
	except KeyError:
		pass

	if string not in __all__:
		raise ImportError(f"Package 'sphoin.plot' has no class '{string}'")

	module = import_module(_build_path(string), package="sphoin.plot")
	class__ = getattr(module, string)

	__cache[string] = class__
	return class__

def intersperse(_list: list, item):
	result = [item] * (len(_list) * 2 -1)
	result[0::2] = _list
	return result


def test():

	from ..app import Slot
	from ._plot import Plot
	from rich.console import Console

	console = Console()

	slot = Slot.from_config()

	close = Plot.line(
		series=slot.ohlcv.close,
		signals=slot.signals_sum,
		min_flag=slot.min_flag,
		max_flag=slot.max_flag, 
		color1=slot.colors['1'], 
		color2=slot.colors['-1'],
		sizeX=console.size.width,
		sizeY=20,
		signal_as_line=True,
		dark_theme=False,
		brightness=False)
	console.print(close)

	time = Plot.time_bar(slot=slot,sizeX=console.size.width,dark_theme=False)
	console.print(time)

	signals = Plot.line(
		series=slot.signals_sum,
		signals=slot.signals_sum,
		min_flag=slot.min_flag,
		max_flag=slot.max_flag, 
		color1=slot.colors['1'], 
		color2=slot.colors['-1'],
		sizeX=console.size.width,
		sizeY=slot.nr_studies*2+1,
		signal_as_line=False,
		dark_theme=True,
		brightness=True)
	console.print(signals)

	signal_as_line = True

	for study in slot.studies:

		series = study.config.indicator_1.raw
		if study.config.indicator_2 != None:
			series = [series]
			series.append(study.config.indicator_2.raw)

		s = Plot.line(
			series=series,
			signals=study.result,
			min_flag=0,
			max_flag=0,
			color1=slot.colors['1'], 
			color2=slot.colors['-1'],
			sizeX=console.size.width,
			sizeY=20,
			dark_theme=True,
			signal_as_line=signal_as_line,
			brightness=not signal_as_line)

		signal_as_line = not signal_as_line

		console.print(s)
		s_bar = Plot.study_bar(
			study=study,
			color1=slot.colors['1'],
			color2=slot.colors['-1'],
			sizeX=console.size.width,
			dark_theme=True,
			brightness=signal_as_line
			)
		console.print(s_bar)


from textual.reactive import reactive
from textual.widgets import Static
from rich.text import Text, Span
from rich.style import Style
from rich.color import Color
from ...app import Slot
from ...plot import Plot
from ...utils import footer, text_with_style, uni_chars
import datetime

class Footer(Static):

	slot : [Slot, None] = reactive(None)
	banner : list = reactive([Text("Painting..")])
	brightness : bool = reactive(False)

	def __init__(self, slot: Slot) -> None:
		self._slot = slot
		super().__init__()

	def paint_banner(self, slot: Slot, brightness: bool) -> list:
		if self.size.width <50:
			return [Text()]
		if slot!=None:
			bright = "bright_" if brightness else ""
			eta = datetime.timedelta(seconds=int(slot.ETA))
			formatted = {
			"exchange":slot.exchange,
			"market":slot.market,
			"interval":slot.interval,
			"max_flag":slot.max_flag,
			"min_flag":slot.min_flag
			}
			color = footer['color']
			color['max_flag']['bgcolor'] = slot.colors['1']
			color['min_flag']['bgcolor'] = slot.colors['-1']
			sizes = {
				"left":{
					"s":Text(footer["display"]["left"]["s"].format(**formatted)),
					"m":Text(footer["display"]["left"]["m"].format(**formatted)),
					"l":Text(footer["display"]["left"]["l"].format(**formatted)),
					"xl":Text(footer["display"]["left"]["xl"].format(**formatted))
				},
				"mid":{
					"s":Text(footer["display"]["mid"]["s"]),
					"m":Text(footer["display"]["mid"]["m"]),
					"l":Text(footer["display"]["mid"]["l"]),
					"xl":Text(footer["display"]["mid"]["xl"]),
				},
				"right":{
					"s":Text(footer["display"]["right"]["s"].format(eta=eta)),
					"m":Text(footer["display"]["right"]["m"].format(eta=eta)),
					"l":Text(footer["display"]["right"]["l"].format(eta=eta)),
					"xl":Text(footer["display"]["right"]["xl"].format(eta=eta))
				}
			}
			size = ["xl","l","m","s"]
			for r in size:
				tright = sizes["right"][r]
				tright.stylize(Style(color="black",bgcolor=f"{bright}white"))
				right = len(tright)
				for l in size:
					tleft = sizes["left"][l]
					spans = []
					for key, value in formatted.items():
						_values = [f" {value} |",f"{value} |",f"{value}|",f"{value}"]
						for value in _values:
							exists = value in str(tleft)
							if exists:
								start = str(tleft).index(str(value))
								end = str(tleft).index(str(value))+len(str(value))
								spans.append(Span(start=start,end=end,style=Style(color=color[key]['color'],bgcolor=f"{bright}{color[key]['bgcolor']}")))
								break
					tleft.spans = spans
					left = len(tleft)				
					for m in size:
						tmid = sizes["mid"][m]
						for action in footer['actions'].keys():
							for key in footer["actions"][action]:
								if key in tmid:
									tmid.stylize(Style(color="black",bgcolor=f"{bright}yellow"))
									tmid.stylize(Style.on(click=action),start=str(tmid).index(key),end=str(tmid).index(key)+len(key))
									break
						mid = len(tmid)
						spare = self.size.width - right - left - mid
						if spare >= 0:
							return [tleft,Text.from_ansi(uni_chars["space"]*spare),tmid,tright]
		else:
			return [Text()]

	def on_mount(self) -> None:
		self.slot = self._slot
		self.styles.link_background = "yellow"
		self.styles.link_style = "bold"
		self.styles.link_hover_style = "bold italic"
		self.styles.link_hover_background = "magenta"

		self.set_interval(1, self.refresh_banner)

	def refresh_banner(self) -> None:
		self.banner = self.paint_banner(slot=self.slot,brightness=self.brightness)

	def watch_banner(self, banner: list) -> None:
		string = Text()
		for text in banner:
			string += text
		self.update(string)

	def watch_slot(self, slot: Slot) -> None:
		self.banner = self.paint_banner(slot=slot, brightness=self.brightness)

	def watch_brightness(self, brightness: bool) -> None:
		self.banner = self.paint_banner(slot=self.slot, brightness=brightness)







import datetime
from dateutil import tz
from ..models import Ohlcv, Study
from ..app import Slot
from ..utils import isNumber, uni_chars, intersperse, text_with_style
from rich.console import Console
from rich.color import Color, ANSI_COLOR_NAMES
from rich.text import Text
from rich.style import Style
from math import floor, ceil

class Plot:
	"""
Plot module for sphoin.app Slots

Methods
-------
line
    Generate an ascii chart for a series of numbers
study_bar
    Generate an ascii bar chart for a series of numbers
time_bar
    Generate an ascii time bar for a series of numbers
	"""

	def __init__(self):
		pass

	def line(
		series: list,
		signals: list,
		min_flag: int = 0,
		max_flag: int = 0,
		color1: str = 'magenta', 
		color2: str = 'cyan',
		sizeY: int = 200, 
		sizeX: int = 50,
		signal_as_line: bool = False,
		dark_theme: bool = True,
		brightness: bool = True,
		as_ansi: bool = False) -> str:
		"""
Generate an ascii chart for a series

Parameters
----------
series : list
    List of values to chart
signals : list
    List of signals corresponding to each series element
min_flag : int
    Filter signals lower or equal with min_flag
max_flag : int
	Filter signals bigger or equal with max_flag
color1 : str
	rich ansi color name
	Default : magenta
	see 'python -m rich.color'
color2 : str
	rich ansi color name
	Default : cyan
	see 'python -m rich.color'
sizeY : int
	Height of chart plot
	Default : 200
	Constraints
		min : 4
		max : 200
sizeX : int
	Width of chart plot
	Default : 50
	Contraints
		min : 50
		max : 500
signal_as_line : bool
	Plots signal as vertical line (True) or dot (False)
	Default : False
dark_theme : bool
	Theme of chart plot
	Default : True
brightness : bool
	Default : True
as_ansi : bool
	Default : False

Returns
------
rich Text
	Ansi chart
		"""

		if len(series) == 0:
			return Text()
		if len(signals) == 0:
			return Text()

		
		if sizeY < 4:
			sizeY = 4
		elif sizeY > 200:
			sizeY = 200
		if sizeX < 50:
			sizeX = 50
			# crop = -sizeX+11
		elif sizeX > 500:
			sizeX = 500
		crop = -sizeX+11

		sizeY -= 1

		bright = "bright_" if brightness else ""
		line_type = "curved"
		line_color = f"{bright}white" if dark_theme else "black"
		background_color = "black" if dark_theme else f"{bright}white"
		color1 = f"{bright}{color1}"
		color2 = f"{bright}{color2}"

		offset = 1
		if not isinstance(series[0],list):
			if all(not isNumber(n) for n in series):
				return Text()
			else:
				if len(series)+12<=sizeX:
					sizeX = len(series)+12
					crop = 0
				series = [series[crop:]]

		else:
			if len(series[0])+12<=sizeX:
				sizeX = len(series[0])+12
				crop = 0
			line_type = "angular"
			series = [x[crop:] for x in series]

		signals = signals[crop:]
		minimum = min(filter(isNumber, [y for x in series for y in x]))
		maximum = max(filter(isNumber, [y for x in series for y in x]))
		interval = abs(float(maximum) - float(minimum))
		interval = 0.1 if interval == 0 else interval
		ratio = (sizeY) / interval
		intmin2 = int(round(float(minimum) * ratio))
		intmax2 = int(round(float(maximum) * ratio))

		rows = abs(intmax2 - intmin2)

		width = 0
		for i in range(0,len(series)):
			width = max(width, len(series[i]))
		width += offset

		result = [[text_with_style(string=uni_chars["space"],fg=line_color,bg=background_color)] * (width-offset) for i in range(rows+1)]

		for y in range(intmin2, intmax2 + 1):
			try:
				label = '{:8.9f}'.format(float(maximum) - ((y - intmin2) * interval / rows))[:11]
			except Exception as e:
				label = '{:8.9f}'.format(0)
			result[y - intmin2][max(offset - len(label), 0)] = text_with_style(string=label+uni_chars["vertical_bar"],fg=line_color,bg=background_color)

		font_weights =["light","heavy"]
		for i in range(0, len(series)):
			font_weight = font_weights[i]
			for x in range(0, len(series[i])-1):
				try:
					y0 = int(round(series[i][x + 0] * ratio) - intmin2)
					y1 = int(round(series[i][x + 1] * ratio) - intmin2)
				except Exception as e:
					y0 = 0
					y1 = 0
				if y0 == y1:
					horizontal = uni_chars[font_weight][line_type]["horizontal"]
					if i == 1:
						if uni_chars["light"][line_type]["up_right"] in result[rows - y0][x + offset]:
							horizontal = uni_chars["heavy"]["cross"]["horizontal"]["light_up"]
						elif uni_chars["light"][line_type]["left_up"] in result[rows - y0][x + offset]:
							horizontal = uni_chars["heavy"]["cross"]["horizontal"]["light_up"]
						elif uni_chars["light"][line_type]["vertical"] in result[rows - y0][x + offset]:
							horizontal = uni_chars["heavy"]["cross"]["full"]["light_vertical"]
						elif uni_chars["light"][line_type]["down_right"] in result[rows - y0][x + offset]:
							horizontal = uni_chars["heavy"]["cross"]["horizontal"]["light_down"]
						elif uni_chars["light"][line_type]["left_down"] in result[rows - y0][x + offset]:
							horizontal = uni_chars["heavy"]["cross"]["horizontal"]["light_down"]
					paint0 = text_with_style(string=horizontal,fg=line_color,bg=background_color)
					try:#background from signal
						if signals[x]>=max_flag:
							paint0 = text_with_style(string=horizontal,fg="black",bg=color1)
						elif signals[x]<=min_flag:
							paint0 = text_with_style(string=horizontal,fg="black",bg=color2)
					except:
						pass
					result[rows - y0][x + offset] = paint0
				else:
					if y0 > y1:
						going_lower1 = uni_chars[font_weight][line_type]["up_right"]
						if i == 1:
							if uni_chars["light"][line_type]["horizontal"] in result[rows - y1][x + offset]:
								going_lower1 = uni_chars["heavy"]["cross"]["horizontal"]["up_right"]
							elif uni_chars["light"][line_type]["left_up"] in result[rows - y1][x + offset]:
								going_lower1 = uni_chars["heavy"]["cross"]["horizontal"]["up_right"]
							elif uni_chars["light"][line_type]["vertical"] in result[rows - y1][x + offset]:
								going_lower1 = uni_chars["heavy"]["cross"]["vertical"]["up_right"]
							elif uni_chars["light"][line_type]["down_right"] in result[rows - y1][x + offset]:
								going_lower1 = uni_chars["heavy"]["cross"]["vertical"]["up_right"]
							elif uni_chars["light"][line_type]["left_down"] in result[rows - y1][x + offset]:
								going_lower1 = uni_chars["heavy"]["cross"]["full"]["light_left_down"]
						paint1 = text_with_style(string=going_lower1,fg=line_color,bg=background_color)
						try:
							if signals[x]>=max_flag:
								paint1 = text_with_style(string=going_lower1,fg="black",bg=color1)
							elif signals[x]<=min_flag:
								paint1 = text_with_style(string=going_lower1,fg="black",bg=color2)
						except:
							pass
						result[rows - y1][x + offset] = paint1
						
						going_lower2 = uni_chars[font_weight][line_type]["left_down"]
						if i == 1:
							if uni_chars["light"][line_type]["horizontal"] in result[rows - y0][x + offset]:
								going_lower2 = uni_chars["heavy"]["cross"]["horizontal"]["left_down"]
							elif uni_chars["light"][line_type]["left_up"] in result[rows - y0][x + offset]:
								going_lower2 = uni_chars["heavy"]["cross"]["vertical"]["left_down"]
							elif uni_chars["light"][line_type]["vertical"] in result[rows - y0][x + offset]:
								going_lower2 = uni_chars["heavy"]["cross"]["vertical"]["left_down"]
							elif uni_chars["light"][line_type]["down_right"] in result[rows - y0][x + offset]:
								going_lower2 = uni_chars["heavy"]["cross"]["horizontal"]["left_down"]
							elif uni_chars["light"][line_type]["up_right"] in result[rows - y0][x + offset]:
								going_lower2 = uni_chars["heavy"]["cross"]["full"]["light_up_right"]
						paint2 = text_with_style(string=going_lower2,fg=line_color,bg=background_color)
						try:
							if signals[x]>=max_flag:
								paint2 = text_with_style(string=going_lower2,fg="black",bg=color1)
							elif signals[x]<=min_flag:
								paint2 = text_with_style(string=going_lower2,fg="black",bg=color2)
						except:
							pass
						result[rows - y0][x + offset] = paint2
					
					elif y0 < y1:
						going_up1 = uni_chars[font_weight][line_type]["down_right"]
						if i == 1:
							if uni_chars["light"][line_type]["horizontal"] in result[rows - y1][x + offset]:
								going_up1 = uni_chars["heavy"]["cross"]["horizontal"]["down_right"]
							elif uni_chars["light"][line_type]["left_up"] in result[rows - y1][x + offset]:
								going_up1 = uni_chars["heavy"]["cross"]["full"]["light_left_up"]
							elif uni_chars["light"][line_type]["vertical"] in result[rows - y1][x + offset]:
								going_up1 = uni_chars["heavy"]["cross"]["vertical"]["down_right"]
							elif uni_chars["light"][line_type]["up_right"] in result[rows - y1][x + offset]:
								going_up1 = uni_chars["heavy"]["cross"]["vertical"]["down_right"]
							elif uni_chars["light"][line_type]["left_down"] in result[rows - y1][x + offset]:
								going_up1 = uni_chars["heavy"]["cross"]["horizontal"]["down_right"]
						paint1 = text_with_style(string=going_up1,fg=line_color,bg=background_color)
						try:
							if signals[x]>=max_flag:
								paint1 = text_with_style(string=going_up1,fg="black",bg=color1)
							elif signals[x]<=min_flag:
								paint1 = text_with_style(string=going_up1,fg="black",bg=color2)
						except:
							pass
						result[rows - y1][x + offset] = paint1
						
						going_up2 = uni_chars[font_weight][line_type]["left_up"]
						if i == 1:
							if uni_chars["light"][line_type]["horizontal"] in result[rows - y0][x + offset]:
								going_up2 = uni_chars["heavy"]["cross"]["horizontal"]["left_up"]
							elif uni_chars["light"][line_type]["down_right"] in result[rows - y0][x + offset]:
								going_up2 = uni_chars["heavy"]["cross"]["full"]["light_down_right"]
							elif uni_chars["light"][line_type]["vertical"] in result[rows - y0][x + offset]:
								going_up2 = uni_chars["heavy"]["cross"]["vertical"]["left_up"]
							elif uni_chars["light"][line_type]["up_right"] in result[rows - y0][x + offset]:
								going_up2 = uni_chars["heavy"]["cross"]["horizontal"]["left_up"]
							elif uni_chars["light"][line_type]["left_down"] in result[rows - y0][x + offset]:
								going_up2 = uni_chars["heavy"]["cross"]["vertical"]["left_up"]
						paint2 = text_with_style(string=going_up2,fg=line_color,bg=background_color)
						try:
							if signals[x]>=max_flag:
								paint2 = text_with_style(string=going_up2,fg="black",bg=color1)
							elif signals[x]<=min_flag:
								paint2 = text_with_style(string=going_up2,fg="black",bg=color2)
						except:
							pass
						result[rows - y0][x + offset] = paint2

					start = min(y0, y1) + 1
					end = max(y0, y1)
					for y in range(start, end):
						vertical = uni_chars[font_weight][line_type]["vertical"]
						if i == 1:
							if uni_chars["light"][line_type]["up_right"] in result[rows - y][x + offset]:
								vertical = uni_chars["heavy"]["cross"]["vertical"]["light_right"]
							elif uni_chars["light"][line_type]["left_up"] in result[rows - y][x + offset]:
								vertical = uni_chars["heavy"]["cross"]["vertical"]["light_left"]
							elif uni_chars["light"][line_type]["horizontal"] in result[rows - y][x + offset]:
								vertical = uni_chars["heavy"]["cross"]["full"]["light_horizontal"]
							elif uni_chars["light"][line_type]["down_right"] in result[rows - y][x + offset]:
								vertical = uni_chars["heavy"]["cross"]["vertical"]["light_right"]
							elif uni_chars["light"][line_type]["left_down"] in result[rows - y][x + offset]:
								vertical = uni_chars["heavy"]["cross"]["vertical"]["light_left"]
						paint3 = text_with_style(string=vertical,fg=line_color,bg=background_color)
						try:
							if signals[x]>=max_flag:
								paint3 = text_with_style(string=vertical,fg="black",bg=color1)
							elif signals[x]<=min_flag:
								paint3 = text_with_style(string=vertical,fg="black",bg=color2)
						except:
							pass
						result[rows - y][x + offset] = paint3

		if signal_as_line:
			for j in range(len(result[0])):
				col = [row[j] for row in result]
				if any(color1 in x[1] for x in col):
					for i in range(len(result)):
						if background_color in result[i][j][1]:
							result[i][j] = text_with_style(string=result[i][j][0],fg="black",bg=color1)
				elif any(color2 in x[1] for x in col):
					for i in range(len(result)):
						if background_color in result[i][j][1]:
							result[i][j] = text_with_style(string=result[i][j][0],fg="black",bg=color2)

		ret = []
		for row in result:
			r = []
			for item in row:
				r.append(tuple(item))
			ret.append(Text.assemble(*tuple(r),end=""))
		ret = intersperse(ret,Text("\n"))
		ret = Text.assemble(*tuple(ret))
		if as_ansi:
			console = Console()
			with console.capture() as capture:
				console.print(ret)
			ret = capture.get()
		return ret

	def study_bar(
		study: Study,
		color1: str = 'magenta', 
		color2: str = 'cyan', 
		sizeX: int = 50,
		dark_theme: bool = True,
		brightness: bool = True,
		as_ansi: bool = False) -> Text:
		"""
Generate an ascii bar chart for Sphoin Slot Study

Parameters
----------
study : Study
    Sphoin Slot Study
    Example:
    	slot = Slot(config='example')
    	study = slot.studies[0]
color1 : str
	rich ansi color name
	Default : 'magenta'
	see 'python -m rich.color'
color2 : str
	rich ansi color name
	Default : 'cyan'
	see 'python -m rich.color'
sizeX : int
	Width of chart plot
	Default : 50
	Contraints
		min : 50
		max : 500
dark_theme : bool
	Theme of chart plot
	Default : True
brightness : bool
	Default : True
as_ansi : bool
	Default : False
Returns
------
rich Text
	Ansi chart
		"""
		bright = "bright_" if brightness else ""
		if sizeX < 50:
			sizeX = 50
		elif sizeX > 500:
			sizeX = 500
		crop = -sizeX+11
		if len(study.result)+12<=sizeX:
			sizeX = len(study.result)+12
			crop = 0
		line_color = f"{bright}white" if dark_theme else "black"
		background_color = "black" if dark_theme else f"{bright}white"
		vv = 8 if bright else 0
		bgstudy = [k for k,v in ANSI_COLOR_NAMES.items() if v-vv==int(study.code.split("#")[1])][0]
		inter = study.study.center(11,uni_chars["space"]) + uni_chars["light"]["angular"]["vertical"]
		inter = [Text.assemble(tuple(text_with_style(string=inter,fg='black',bg=bgstudy)))]
		series = study.result[crop:]
		for i in range(0,len(series)-1):
			if series[i] == None or int(series[i])==0:
				inter.append(text_with_style(string=uni_chars['space'],fg=line_color,bg=background_color))
			elif int(series[i]) == 1:
				inter.append(text_with_style(string=uni_chars['space'],fg=line_color,bg=f"{bright}{color1}"))
			elif int(series[i]) == -1:
				inter.append(text_with_style(string=uni_chars['space'],fg=line_color,bg=f"{bright}{color2}"))
		ret = Text.assemble(*tuple(inter))
		if as_ansi:
			console = Console()
			with console.capture() as capture:
				console.print(ret)
			ret = capture.get()
		return ret

	def time_bar(
		slot: Slot, 
		sizeX: int = 50,
		dark_theme: bool = True,
		brightness: bool = True,
		as_ansi: bool = False) -> str:
		"""
Generate an ascii time bar for a Sphoin Slot

Parameters
----------
slot : Slot
    Sphoin Slot 
    Example :
    	slot = Slot(config='example')
sizeX : int
	Width of chart plot
	Default : 50
	Contraints
		min : 50
		max : 500
dark_theme : bool
	Theme of chart plot
	Default : True
brightness : bool
	Default : True
as_ansi : bool
	Default : False

Returns
------
rich Text
	Ansi chart
		"""
		if sizeX < 50:
			sizeX = 50
		elif sizeX > 500:
			sizeX = 500
		crop = -sizeX+11
		if len(slot.ohlcv.date)+12<=sizeX:
			sizeX = len(slot.ohlcv.date)+12
			crop = 0
		from_zone = tz.tzutc()
		to_zone = tz.tzlocal()
		bright = "bright_" if brightness else ""
		line_color = f"{bright}white" if dark_theme else "black"
		background_color = "black" if dark_theme else f"{bright}white"
		series = slot.ohlcv.date[crop:]
		result = []	
		visible_length = divmod(len(series),6)
		indices = [6*x+2 for x in range(visible_length[0])]
		for index in indices:
			try:
				current_date = datetime.datetime.strptime(series[-index-1], "%Y-%m-%dT%H:%M:%S")
			except Exception as e:
				current_date = datetime.datetime.strptime(series[-index-1].split(".")[0], "%Y-%m-%d %H:%M:%S")
			current_date = current_date.replace(tzinfo=from_zone)
			current_date = current_date.astimezone(to_zone)
			if slot.interval[-1] in ["m","h"]:
				current_date = current_date.strftime("%H:%M")
			else:
				current_date = current_date.strftime("%d/%m")
			date = text_with_style(string=current_date.rjust(6,' '),fg=line_color,bg=background_color)
			result.insert(0,date)
		_times = Text.assemble(*tuple(result))
		_label = Text.assemble(tuple(text_with_style(string='time'.rjust(12,'·'),fg="black",bg=f"{bright}yellow")))
		_add = sizeX - len(_times) - len(_label)
		if _add<0:
			result[0][0] = result[0][0][abs(_add):]
			_times = Text.assemble(*tuple(result))
		elif 0<_add<6:
			if crop==0:
				_add -= 1
			result.insert(0,text_with_style(string=uni_chars['space']*_add,fg=line_color,bg=background_color))
			_times = Text.assemble(*tuple(result))
		ret = Text.assemble(_label,_times)
		if as_ansi:
			console = Console()
			with console.capture() as capture:
				console.print(ret)
			ret = capture.get()
		return ret




__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

from textual.app import App, ComposeResult
from textual.containers import Container, Vertical
from textual.widgets import Static
from rich.color import Color
from ..app import Slot
from ..plot import Plot
from .widgets import Graph, Footer, Time, Help


class Body(Container):
	pass

class TUI(App[None]):

	CSS = """
Screen {
	align: center middle;
	background: $surface;
	overflow: hidden;
	layers: base overlay notes notifications;
}

Graph {
	width: 100%;
	height: 100%;
}

Body {
	height: 100%;
	width: 100%;
}

Help {
	width: 80%;
	dock: right;
	layer: overlay;
}

Help:focus-within {
	offset: 0 0 !important;
}

Help.-hidden {
	offset-x: 100%;
}

Help Title {
    background: $surface;
    color: $secondary;
    padding: 2 4;
    border-right: vkey $background;
    dock: top;
    text-align: center;
    text-style: bold;
}

#line {
	width: 100%;
	height: auto;
	dock: top;
}

#signals {
	width: 100%;
	height: auto;
}

#time {
	width: 100%;
	height: auto;
}

#studies {
	width: 100%;
	height: auto;
}

#footer {
	width: 100%;
	height: auto;
	dock: bottom;
}
"""

	BINDINGS = [
		("r","refresh","Refresh"),
		("h","help","Help"),
		("q","quit","Quit"),
		("t", "theme", "Theme"),
		("s", "signal","Signal"),
		("b", "brightness","Brightness")
	]

	def __init__(self, **kwargs) -> None:
		if 'slot' in kwargs.keys():
			self.slot = kwargs['slot']
		else:
			self.slot = Slot(config='example')
		super().__init__()			

	def compose(self) -> ComposeResult:
		plots = []
		for layout in self.slot.layout:
			if layout in ['line','signals','studies']:
				widget = Graph(slot_init=self.slot, plot_type=layout)
			elif layout == 'time':
				widget = Time(slot=self.slot)
			elif layout == 'footer':
				widget = Footer(slot=self.slot)
			plots.append(
				Container(
					widget,
					id=layout
				)
			)
		yield Container(
			Help(classes="-hidden"),
			Body(
				*tuple(plots),
				id="contents"
				)
			)

	def on_mount(self) -> None:
		self.set_interval(1,self.refresh_graphs)
		self.compute_height()

	def on_resize(self) -> None:
		self.compute_height()

	def refresh_graphs(self) -> None:
		self.slot.ETA = self.slot.ETA-1
		if self.slot.ETA == 0:
			self.slot.refresh()
			# if self.slot.latest_signal == 0:
			# 	self.app.bell()
		
	def action_refresh(self) -> None:
		self.slot.refresh()

	def action_help(self) -> None:
		sidebar = self.query_one(Help)
		self.set_focus(None)
		if sidebar.has_class("-hidden"):
			sidebar.remove_class("-hidden")
		else:
			if sidebar.query("*:focus"):
				self.screen.set_focus(None)
			sidebar.add_class("-hidden")

	def action_brightness(self) -> None:
		for widget in [Graph,Time,Footer]:
			w = self.query(widget)
			if len(w)>0:
				for ww in w:
					ww.brightness = not ww.brightness

	def action_theme(self) -> None:
		self.dark = not self.dark
		for widget in [Graph,Time]:
			w = self.query(widget)
			if len(w)>0:
				for ww in w:
					ww.dark = self.dark

	def action_signal(self) -> None:
		graphs = self.query(Graph)
		for graph in graphs:
			graph.signal_as_line = not graph.signal_as_line
		
	def compute_height(self) -> None:
		_studies = self.slot.nr_studies if "studies" in self.slot.layout else 0
		_studies = 4 if self.slot.nr_studies <= 4 else _studies
		_signals = self.slot.nr_studies if "signals" in self.slot.layout else 0
		_signals = 4 if self.slot.nr_studies <= 4 else _signals
		_time = 1 if "time" in self.slot.layout else 0
		_footer = 1 if "footer" in self.slot.layout else 0

		height = {
			"line": self.size.height-_studies-_signals-_time-_footer,
			"studies": _studies,
			"signals": _signals,
			"time": _time,
			"footer": _footer
		}

		for layout in self.slot.layout:
			graphs = self.query(f"#{layout}")
			if len(graphs)>0:
				graphs.first().styles.height = height[layout]


if __name__ == "__main__":
	slot = Slot(config="example")
	app = TUI(slot=slot)
	app.run()




from textual.widgets import MarkdownViewer, Static
from textual.containers import Container
from textual.app import ComposeResult
from textual.reactive import reactive
from rich.text import Text

md = """\
# TUI key bindings

* h - Help sidebar
* r - Refresh Slot data
* t - Toggle theme
* s - Signals theme
* b - Toggle brightness
* q - Quit

# Installation

## Stable release
To install sphoin, run this command in your terminal:

```
pip install sphoin
```

This is the preferred method to install sphoin, as it will always install the most recent stable release.

## From sources

The sources for sphoin can be downloaded from the `Github repo`.
* clone the public repository
```
git clone https://github.com/Parsecom/sphoin
```
* install from source
```
python setup.py install
```

## Docker
```
docker pull pom11/sphoin
```

## `.yaml` config file
```yaml
---
uid: 111YOURUID111
api-key: 111YOURAPIKEY111
api-secret: 111YOURSECRET111
layout:
  - line
  - time
  - signals
  - studies
  - footer
  
```

In `layout` specifies what `Plot`s should be displayed

* `line` - Price graph
* `time` - Time bar
* `signals` - Signals sum graph
* `studies` - Studies bars
* `footer` - Footer with Slot details

# Usage

## CLI
```
sphoin --help
```
View example
```
sphoin --example
```
Load config file
```
sphoin --config config.yaml
```
Print Plot test
```
sphoin --plot
sphoin.plot
```

## Docker
Pull image from docker hub
```shell
docker pull pom11/sphoin
```
To run sphoin TUI with `config` file in docker you need to mount local directory to `/usr/src/app`
```shell
docker run -v ${PWD}/config.yaml:/usr/src/app/config.yaml -it pom11/sphoin -c config.yaml
``` 


## Python

Import package
```python
from sphoin.app import Slot
```
### Slot from keys
```python
my_slot = Slot.from_keys(uid="111YOURUID111",
      api_key="111YOURAPIKEY111",
      api_secret="111YOURSECRET111")
```
### Slot from config file
```python
my_slot = Slot.from_config(file="/absolute/path/of/config.yaml")
```
### Slot from dict
Obtain dict from `https://api.sphoin.app/api/v1/data/json` endpoint
```python
import requests

url = "https://api.sphoin.app/api/v1/data/json"

payload = "{\n    \"uid\": \"111YOURUID111\",\n    \"api-key\": \"111YOURAPIKEY111\",\n    \"api-secret\": \"111YOURSECRET111\"\n}"
headers = {
  'X-sphoin.app': 'LIadeCyi15FAwoRBkiu9fJsYPvWecSxb'
}

response = requests.request("POST", url, headers=headers, data=payload)

json = response.json()
```
Init Slot from obtained result
```python
my_slot = Slot.from_dict(data=json)
```
### Inspect Slot
```python
from rich import print
from rich import inspect

print(my_slot.json)
# insepct slot attributes
print(my_slot.__doc__)
print(inspect(my_slot,all=True))
```

"""

class Title(Static):
	pass


class Help(Container):

	def compose(self) -> ComposeResult:
		yield Title(Text.from_markup("[bold cyan]sphoin[/][bold magenta].app[/] [bold yellow]TUI[/]"))
		yield MarkdownViewer(md, show_table_of_contents=True)



__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
	from ._footer import Footer
	from ._graph import Graph
	from ._time import Time
	from ._help import Help

__all__ = [
	"Footer",
	"Graph",
	"Time",
	"Help"
]

__cache = {}

def __getattr__(string):
	try:
		return __cache[string]
	except KeyError:
		pass

	if string not in __all__:
		raise ImportError(f"Package 'sphoin.tui.widgets' has no class '{string}'")

	module = import_module(f"._{string.lower()}", package="sphoin.tui.widgets")
	class__ = getattr(module, string)

	__cache[string] = class__
	return class__


class Ohlcv:
	"""
Ohlcv class

Attributes
----------
json : dict
date : list
	List of dates
open : list
	List of open values
high : list
	List of high values
low : list
	List of low values	
close : list
	List of close values
volume : list
	List of volume values
	"""
	def __init__(self,**kwargs) -> None:
		self.json = kwargs
		self.date : list = kwargs['date']
		self.open : list = kwargs['open']
		self.high : list = kwargs['high']
		self.low : list = kwargs['low']
		self.close : list = kwargs['close']
		self.volume : list = kwargs['volume']



"""Console script for sphoin."""

__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

import argparse
import sys
from .app import Slot
from .plot import test
from .tui import TUI
from rich.table import Table
from rich import print

app_title = f' [bold cyan]sphoin[/bold cyan][bold magenta].app[/bold magenta] [bold yellow]TUI[/bold yellow]'
table = Table(title=None,box=None)
table.add_column('Argument',justify='left', style='green', no_wrap=False)
table.add_column('',justify='left', style='yellow', no_wrap=False)
table.add_column('Help',justify='left', style='bold cyan', no_wrap=False)
data = [
		['--help','-h', 'Print help'],
		['--example','-e','Start TUI with example configuration'],
		['--config','-c','Load .yaml config file'],
		['--plot','-p','Test Plot or type sphoin.plot']
	]
for d in data:
	table.add_row(*d)

def main():
	"""Console script for sphoin."""
	print(app_title)
	import argparse
	parser = argparse.ArgumentParser(description="Connect via apiv1 to sphoin.app slots",add_help=False)
	group = parser.add_mutually_exclusive_group()
	group.add_argument("-h","--help",help="Print help",action="store_true")
	group.add_argument("-e","--example",help="Start TUI with example configuration",action="store_true")
	group.add_argument("-p","--plot",help="Test Plot or type sphoin.plot",action="store_true")
	group.add_argument("-c","--config",help="Load .yaml config file")
	args = parser.parse_args()

	if len(sys.argv)==1:
		print(table)
		sys.exit(1)

	if args.help:
		print(table)
		sys.exit(1)
	elif args.plot:
		test()
	elif args.config:
		print(f" [bold cyan]Loading[/bold cyan] [yellow]{args.config}[/]...")
		slot = Slot.from_config(file=args.config)
		if slot == "slot_error":
			print(" [red]Unkonwn error[/]")
			sys.exit(1)
		elif slot.status == 'error':
			print(f" [yellow]status[/] : [red]{slot.status}[/]")
			print(f" [red]error[/]: {slot.error}")
			sys.exit(1)
		elif slot.status == 'live':
			TUI(slot=slot).run()
		else:
			print(" [red]Unkonwn error[/]")
			sys.exit(1)
	elif args.example:
		print(" [bold cyan]Loading...[/bold cyan]")
		slot = Slot.from_config()
		if slot == "slot_error":
			print(" [red]Unkonwn error[/]")
			sys.exit(1)
		elif slot.status == 'error':
			print(f" [yellow]status[/] : [red]{slot.status}[/]")
			print(f" [red]error[/]: {slot.error}")
			sys.exit(1)
		elif slot.status == 'live':
			TUI(slot=slot).run()
		else:
			print(" [red]Unkonwn error[/]")
			sys.exit(1)
		
	

if __name__ == "__main__":
	sys.exit(main())



"""Main module."""

__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

import requests
import json
import yaml
import time
import sys,select
import os
import datetime as dt
from sphoin.models import Ohlcv, Study

class Slot:
	"""
Slot class to represent a sphoin.app Slot

Example config.yaml input file
```yaml
---
uid: 111YOURUID111
api-key: 111YOURAPIKEY111
api-secret: 111YOURSECRET111
layout:
  - line
  - time
  - signals
  - studies
  - footer
```


Attributes
----------
status : str
	Slot status
error : str
	Slot error
api_key : str
	Api Key
api_secret : str
	Api Secret
uid : str
	User id
exchange : str
	Exchange
market : str
	Market
interval : str
	Interval
min_flag : int
	Filter signals lower or equal with min_flag
max_flag : int
	Filter signals bigger or equal with max_flag
ETA : int
	Seconds till next update
latest_signal : int
	Latest signal
signals_sum : list
	List of ints with sum of all signals
colors : dict
	Dict with rich color names corresponding to -1 and 1 signals
nr_studies : int
	Number of configured Sphoin Studies
studies : list
	List of configured sphoin Studies

layout : list
	TUI layout config
	Expected values: line, studies, signals

Methods
------
refresh
	Manually fetches Slot result

Returns
-------
Slot
	"""
	def __init__(self) -> None:
		self.show_banner : bool = False
		self.base_url : str = "https://api.sphoin.app/api/v1"

	def refresh(self):
		"""
Updates Slot attributes with new values
		"""
		self.__get_json()

	def __get_screenSize(self, data: dict = {}):
		"""
Get console size from dict

Parameters
----------
data : dict
		"""
		if "sizeX" in data.keys():
			self.sizeX : int = data['sizeX']
		else:
			self.sizeX : int = None
		if "sizeY" in data.keys():
			self.sizeY : int = data['sizeY']
		else:
			self.sizeY : int = None

	@classmethod
	def from_config(cls, file: str = 'example') -> "Slot":
		"""
Init Slot from config file

Example config.yaml input file
```yaml
---
uid: 111YOURUID111
api-key: 111YOURAPIKEY111
api-secret: 111YOURSECRET111
layout:
  - line
  - time
  - signals
  - studies
  - footer
```
Example
	my_slot = Slot.from_config(file='/aboslute/path/of/config.yaml')

Parameters
----------
file : str
	Absolute path of yaml config file
	Default : example
		Inits Slot with example configuration

Returns
-------
Slot
		"""
		if not file:
			raise ValueError("config")
		slot = cls()
		slot.config = file
		if file != "example":
			with open(file,"r") as f:
				data = yaml.safe_load(f)
		else:
			data = {
				"uid" : "111YOURUID111",
				"api-secret" : "111YOURSECRET111",
				"api-key":"111YOURAPIKEY111",
				"layout" : [
					"line",
					"time",
					"studies",
					"signals",
					"footer"
					]
				}
		slot.api_key : str = data["api-key"]
		slot.api_secret : str = data["api-secret"]
		slot.uid : str = data["uid"]
			
		try:
			if not isinstance(data["layout"],list):
				raise ValueError("layout not list")
			elif not any([x in data["layout"] for x in ['line','studies','signals','time','footer']]):
				raise ValueError("layout expected values: line, studies, signals, time, footer")
			else:
				slot.layout : list = data["layout"]
		except Exception as e:
			slot.layout = [
					"line",
					"time",
					"studies",
					"signals",
					"footer"
					]

		slot.__get_screenSize(data)
		slot.__get_json()
		return slot
		
	@classmethod
	def from_keys(cls, api_key: str = None, api_secret: str = None, uid: str = None, layout: list = None, sizeX: int = None, sizeY: int = None) -> "Slot":
		"""
Init Slot from keys

Example:
	my_slot = Slot.from_keys(api_key='111YOURAPIKEY111',api_secret='111YOURSECRET111',uid='111YOURUID111')

Parameters
----------
api_key : str
	Api Key
secret_key : str
	Secret Key
uid : str
	User id
layout : list
	TUI layout
sizeX : int
	Width of console
sizeY : int
	Height of console

Returns
-------
Slot
		"""
		slot = cls()
		slot.config = None
		if not api_key:
			raise ValueError("api_key")
		if not api_secret:
			raise ValueError("api_secret")
		if not uid:
			raise ValueError("uid")
		slot.api_key : str = api_key
		slot.api_secret : str = api_secret
		slot.uid : str = uid
		try:
			if layout:
				if not isinstance(kwargs["layout"],list):
					raise ValueError("layout not list")
				elif not any([x in kwargs["layout"] for x in ['line','studies','signals','time','footer']]):
					raise ValueError("layout list expected values: line, studies, signals, time, footer")
				else:
					slot.layout : list = kwargs["layout"]
			else:
				raise ValueError("layout list missing")
		except Exception as e:
			slot.layout = [
					"line",
					"time",
					"studies",
					"signals",
					"footer"
					]

		slot.__get_screenSize({"sizeX":sizeX,"sizeY":sizeY})
		slot.__get_json()
		return slot

	@classmethod
	def from_dict(cls, data: dict = None) -> "Slot":
		"""
Init Slot from dict

Parameters
----------
data : dict

Returns
-------
Slot
		"""
		if not json:
			raise ValueError("json")
		slot = cls()
		slot.__set_attr(data)
		return slot

	def __get_json(self):
		"""
Gets Slot data from sphoin.app
		"""
		url = self.base_url+f"/data/json"
		headers = {
		'X-sphoin.app': 'LIadeCyi15FAwoRBkiu9fJsYPvWecSxb',
		'Content-Length': '0'}
		payload = json.dumps({
		  "uid": self.uid,
		  "api-key": self.api_key,
		  "api-secret": self.api_secret
		})
		response = requests.request("POST", url, headers=headers, data=payload)
		data =  json.loads(response.text)
		if "slot_error" not in list(data):
			self.__set_attr(data)
		else:
			return TypeError("Error\nYou might want to check Pro Slot config from app")
			sys.exit(1)

	def __set_attr(self, data: dict) -> None:
		"""
Set attributes
		"""		
		if "status" in list(data):
			self.status = data['status']
			self.error = data['error']
		else:
			self.status = 'live'
			self.error = None
			self.exchange : str = data["sphoin.slot"]["exchange"]
			self.market : str = data["sphoin.slot"]["market"]
			self.interval : str = data["sphoin.slot"]["interval"]
			self.max_flag : str = int(data["sphoin.slot"]["flag"].split('|')[0])
			self.min_flag : str = int(data["sphoin.slot"]["flag"].split('|')[1])
			self.ETA : int = int(data["sphoin.slot"]["ETA"])
			self.ohlcv : Ohlcv = Ohlcv(**data["sphoin.slot"]['ohlcv'])
			self.signals_sum : list = data["sphoin.slot"]['sum']
			self.ETA : int = data["sphoin.slot"]['ETA']
			self.latest_signal : int = data["sphoin.slot"]['latest_signal']
			self.colors = data["sphoin.slot"]['colors']
			if 'studies' in data["sphoin.slot"].keys():
				self.studies : list = [Study(**x) for x in data["sphoin.slot"]['studies']]
				self.nr_studies : int = int(len(data["sphoin.slot"]["studies"]))
			elif 'studies.example' in data["sphoin.slot"].keys():
				self.studies : list = [Study(**x) for x in data["sphoin.slot"]['studies.example']]
				self.nr_studies : int = int(len(data["sphoin.slot"]["studies.example"]))
		self.json = data




#!/usr/bin/env python

__author__ = "pom11"
__copyright__ = "Copyright 2023, Parsec Original Mastercraft S.R.L."
__license__ = "MIT"
__version__ = "2.0.8"
__maintainer__ = "pom11"
__email__ = "office@parsecom.ro"

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
	readme = readme_file.read()

requirements = ["requests==2.28.1","pyyaml==6.0","textual==0.15.1","rich==13.3.3","python-dateutil==2.8.2","linkify-it-py==2.0.0"]

setup_requirements = [ ]


setup(
	author="P.O.M.",
	author_email='office@parsecom.ro',
	python_requires='>=3.6',
	classifiers=[
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Natural Language :: English',
		'Programming Language :: Python :: 3.6',
		'Programming Language :: Python :: 3.7',
		'Programming Language :: Python :: 3.8',
		'Programming Language :: Python :: 3.9',
		'Programming Language :: Python :: 3.10',
		'Programming Language :: Python :: 3.11',
	],
	description="Connect to sphoin.app Pro Slots",
	entry_points={
		'console_scripts': [
			'sphoin=sphoin.cli:main',
			'sphoin.plot=sphoin.plot:test'
		],
	},
	install_requires=requirements,
	license="MIT license",
	long_description=readme,
	long_description_content_type="text/markdown",
	include_package_data=True,
	keywords='sphoin',
	name='sphoin',
	packages=find_packages(include=['sphoin', 'sphoin.*']),
	setup_requires=setup_requirements,
	url='https://github.com/Parsecom/sphoin',
	version='2.0.8',
	zip_safe=True,
)